import time
import random
from collections import deque

class State:
    def __init__(self, name, description):
        self.name=name
        self.description=description
        self.edges=[]

class Property:
    def __init__(self, counter, initial_time, health):
        self.counter=counter
        self.initial_time=initial_time
        self.health=health


class Edge:
    def __init__(self, from_node, commands, to_node, description, health, constraints=None):
        self.from_node=from_node
        self.commands=commands
        self.to_node=to_node
        self.description=description
        self.health=health
        self.constraints=constraints
        
class Backpack:
    def __init__(self,max_weight,value):
        self.max_weight=max_weight
        self.value=value


class Object:
    def __init__(self, original_location, object_name, short_description, long_description, weight):
        self.original_location=original_location
        self.object_name=object_name
        self.short_description=short_description
        self.long_description=long_description
        self.weight=weight

class Constraint:
    def __init__(self, ID, object_name, location):
        self.ID=ID
        self.object_name=object_name
        self.location=location




class Engine:
    def __init__(self, filename):
        self.filename=filename
        self.initial_state=None 

    def expand(self,current_room_name):
        connected_room_list=[]
        for i in self.states[current_room_name].edges:
            print i
            connected_room_list.append(i.to_node)
	return connected_room_list


    def backtrace(self, parents,name):
	if name not in parents:
		return [name]
	trace = self.backtrace(parents,parents[name])
	trace.append(name)
	return trace

    def path_search(self, start_room_name, target_room_name):
	if start_room_name == target_room_name:
		return []
	parents={start_room_name:"Start from here-->"}
	todo=deque()
	todo.append(start_room_name)
	while len(todo)>0:
            parents_copy=parents
	    current_room_name = todo.popleft()
	    next_rooms = self.expand(current_room_name)
	    for next in next_rooms:
		if next not in parents and next != start_room_name:
                    parents[next] = current_room_name
                    if next == target_room_name:
                        return self.backtrace(parents,next)
                    todo.append(next)

    def load(self):
        self.states={}
        edges=[]
        self.objects=[]
        self.constraints=[]
        fp=open(self.filename)
        for line in fp:
            if len(line)==0 or line[0] == '#':
                continue
            items = line.strip().split("::")
            if items[0] =='node':
                s=State(*items[1:])
                if self.initial_state is None:
                    self.initial_state=s
                self.states[items[1]]=s
            elif items[0]=='edge':
                edges.append(Edge(*items[1:]))
                #print Edge(*items[1:]).from_node
            elif items[0]=='object':
                self.objects.append(Object(*items[1:]))#Object line
            elif items[0]=='constraint':
                self.constraints.append(Constraint(*items[1:]))
            elif items[0]=='backpack':
                self.bp=Backpack(*items[1:])
                #print "max_value", int(self.bp.value)
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)
            

    def play(self):
        current_state=self.initial_state
        current_object=None
        initial_time=time.time()
        player=Property(0,initial_time,10)
        self.health=player.health
        weight=0
        backpack_weight=0
        print "Hello! Welcome to this game!Please enter your name here to start the game!"
        name_usr_input=raw_input("Your name> ")
        print "Hello! %s, you are now in the game! Enjoy!" %(name_usr_input)
        while True:
            if self.health<0:
                print "Your health point is below 0 now!"
                print "Game Over!"
                print "Do you want to replay the game?"
                usr_input_replay=raw_input()
                if usr_input_replay=="yes":
                    game=Engine("game.txt")
                    game.load()
                    game.play()
                else:
                    return None
            
            self.counter=player.counter
            self.initial_time=player.initial_time
            current_object=[]
            current_object_name_list=[]
            backpackobjects=[]
            print(current_state.description)
            print "You have spent %d min %d seconds in the game." %((time.time()-player.initial_time)/60,(time.time()-player.initial_time)%60)     
            for object in self.objects:
                if object.original_location == current_state.name:
                    print "There is %s on the floor." %object.object_name
                    print(object.short_description)
                    current_object.append(object)
            command=raw_input("Command> ")
            if command== "pick" and len(current_object)>0:
                print "Which object do you want to pick up?"
                for object in current_object:
                    current_object_name_list.append(object.object_name)
                print current_object_name_list
                usr_input_current_object=raw_input()
                if usr_input_current_object in current_object_name_list:
                    #print "Test:found it on the floor!"
                    for object in current_object:
                        if object.object_name==usr_input_current_object:
                            to_pick_object=object
                            #print "Test:Object found"
                            for object in self.objects:
                                if object.original_location == "backpack":
                                    #print "Test:checking weight"
                                    backpack_weight=backpack_weight+int(object.weight)
                            will_be_backpack_weight=int(to_pick_object.weight)+backpack_weight
                            #print "Test:self.bp.value: %d" %(int(self.bp.value))
                            if will_be_backpack_weight>int(self.bp.value):
                                print "Your backpack is too heavy. It cannot be over %d ying units. Please drop something before picking up %s" %(int(self.bp.value),to_pick_object.object_name)
                                will_be_backpack_weight=0
                                backpack_weight=0
                            else:                    
                                to_pick_object.original_location="backpack"
                                print "You have picked up %s" %to_pick_object.object_name
                else:
                    print "Cannot find the stuff you want."
                       
            if command=="pick" and len(current_object)==0:
                print "There is nothing to pick up."
            if command=="drop":
                print "You have the following stuffs. What do you want to drop here?"
                for object in self.objects:
                    #print "Test:" ,object
                    if object.original_location=="backpack":
                        backpackobjects.append(object.object_name)
                print backpackobjects
                print "Please enter the name of the object that you want to drop here."
                usr_input=raw_input("Object> ")
                if usr_input in backpackobjects:
                    #print "Test:found it in the bag!"
                    for object in self.objects:
                        if object.object_name==usr_input:
                            #print "Test:Object found"
                            object.original_location=current_state.name
                            print "You have successfully dropped %s" % object.object_name
                else:
                    print "Cannot find the stuff you want."
            if command =="quit":
                return None

            if command =="help":
                print "Please enter the room you want to search for."
                search_target=raw_input()
                trace=self.path_search(current_state.name, search_target)
                print trace

            if command=="use":
                print "You have the following stuffs to use. what do you want to use here? Keep in mind that you can only use them once!"
                for object in self.objects:
                    #print "Test:" ,object
                    if object.original_location=="backpack":
                        backpackobjects.append(object.object_name)
                print backpackobjects
                print "Please enter the name of the object that you want to use here."
                usr_input=raw_input("Object> ")
                if usr_input in backpackobjects:
                    #print "Test:found it in the bag!"
                    for object in self.objects:
                        if object.object_name==usr_input:
                            #print "Test:Object found"
                            object.original_location="used"
                            print "You have successfully used %s" % object.object_name
                else:
                    print "Cannot find the stuff you want."

            if command=="look":
                for object in self.objects:
                    if object.original_location == "backpack":
                        backpack_weight=backpack_weight+int(object.weight)
                print "You have the following stuffs with you. The current total weight is %d ying units. What do you want to look at?" %backpack_weight
                for object in self.objects:
                    #print "Test:" ,object
                    if object.original_location=="backpack":
                        backpackobjects.append(object.object_name)
                print backpackobjects
                print "Please enter the name of the object that you want to have a look here."
                usr_input=raw_input("Object> ")
                if usr_input in backpackobjects:
                    #print "Test:found it with you!"
                    for object in self.objects:
                        if object.object_name==usr_input:
                            #print "Test:Object found"
                            print object.long_description
                else:
                    print "Cannot find the stuff you want."
                    
            else:
                for edge in current_state.edges:
                    if command in edge.commands:
                        if edge.constraints ==None:
                            current_state = self.states[edge.to_node]
                            player.counter=player.counter+1
                            self.health=self.health-1
                            print "Your current health point is %d." %(self.health)
                            print "This is the %d th move." %(player.counter)
                            break
                        if edge.constraints !=None:
                            for constraint in self.constraints:
                                #print "Test:constraint in self.constraints", constraint
                                if constraint.ID == edge.constraints and constraint.object_name =="eval":
                                    #print "Test:going into eval"
                                    if eval(constraint.location)==True:
                                        #print "Test:You have gone through the door."
                                        current_state = self.states[edge.to_node]
                                        player.counter=player.counter+1
                                        print "This is the %d th move." %(player.counter)
                                        self.health=self.health-1
                                        print "Your current health point is %d." %(self.health)
                                    if eval(constraint.location)==False:
                                        print "You do not satisfy this condition: %s" %constraint.location
                                if constraint.ID == edge.constraints and constraint.object_name=="exec":
                                    #print "Test:going into exec"
                                    exec constraint.location
                                    current_state=self.states[edge.to_node]
                                    player.counter=player.counter+1
                                    print "This is the %d th move." %(player.counter)
                                    self.health=self.health-1
                                    print "Your current health point is %d." %(self.health)
                                    satisfied=True
                                if constraint.ID == edge.constraints and (constraint.object_name !="eval" or constraint.object_name !="exec"):
                                    satisfied=False
                                    #print "Test:constraint.ID==edge.constraints but not eval/exec"
                                    for object in self.objects:
                                        #print "for object in self.obejcts"
                                        if object.original_location==constraint.location and object.object_name == constraint.object_name :
                                            current_state = self.states[edge.to_node]
                                            player.counter=player.counter+1
                                            self.health=self.health-1
                                            print "Your current health point is %d." %(self.health)
                                            print "This is the %d th move." %(player.counter)
                                            #print "Test:Constraint Satisfied!"
                                            satisfied=True
                                            break
                                    if satisfied !=True:
                                        print "You have to pick up %s and put it into %s before going there." %(constraint.object_name, constraint.location)

                                        
                                

game=Engine("game_presentation.txt")
game.load()
game.play()
